# run from salome;
# first import settings.py
import salome
salome.salome_init()

import SalomePyQt
sgPyQt=SalomePyQt.SalomePyQt()

from salome.geom import geomBuilder
from salome.geom import geomtools

import GEOM
import SALOMEDS

import re, os
###
### tools
###
def report(message):
    print(message)
    if salome.sg.hasDesktop():
        salome.sg.updateObjBrowser()
    sgPyQt.processEvents()

###
### GEOMETRY MODULE
###
geompy = geomtools.getGeompy()

try:
    step_filename = os.path.join(study_dir, input_file)
except NameError:
    report("Settings not found, load settings.py first")
report("Loading step {}".format(step_filename))

raw_step = geompy.ImportSTEP(step_filename, False, True)
geompy.addToStudy(raw_step, "")

report("Processing shape")
step = geompy.ProcessShape(raw_step,
    ["FixShape", "FixFaceSize", "DropSmallEdges", "SameParameter"],
    ["FixShape.Tolerance3d", "FixShape.MaxTolerance3d", "FixFaceSize.Tolerance", "DropSmallEdges.Tolerance3d", "SameParameter.Tolerance3d"],
    ["1e-7", "1", "0.0005", "0.0005", "1e-7"])

report("Extracting objects")
step_shapes = geompy.ExtractShapes(step, geompy.ShapeType["SOLID"], True)
filtered_shapes = []

# add all shapes to study to extract names;
# omit shapes that match exclude_objects
exclude_regex = "|".join(["({})".format(o) for o in exclude_objects])
report("Exclude regex: {}".format(exclude_regex))
exclude_pattern = re.compile(exclude_regex)

for shape in step_shapes:
    geompy.addToStudy(shape, "")
    shape_name = shape.GetName()

    if re.match(exclude_pattern, shape_name):
        report("Excluding solid: {} {}".format(shape_name, exclude_regex))
        continue

    report("Added solid {}".format(shape_name))
    shape.internal_name = shape_name
    filtered_shapes.append(shape)

# create a box for domain
if create_domain:
    box_object = geompy.MakeBoxDXDYDZ(box[0], box[1], box[2])
    box_object.internal_name = "box"
    geompy.TranslateDXDYDZ(box_object, box_tr[0], box_tr[1], box_tr[2])
    geompy.addToStudy(box_object, "domain")
    filtered_shapes.append(box_object)
    report("Created domain")
else:
    report("Not creating domain")

# create a partition from all remaining shapes in study_shapes
report("Creating partition")
partition = geompy.MakePartition(filtered_shapes, [], [], [], geompy.ShapeType["SOLID"], 0, [], 0)
geompy.addToStudy(partition, "partition")

report("Restoring subshapes from partition")
partition_subshapes = geompy.ExtractShapes(partition, geompy.ShapeType["SOLID"], True)
partition_subshapes = geompy.RestoreSubShapes(partition)

#for i in range(len(partition_subshapes)):
#    geompy.addToStudyInFather(partition, partition, "")

# join solids with the same name according to grouped_objects
groups = { } # 'name': { 'shapes': [list of shapes], 'group': <group_object> }

def assign_group(shape, shape_name):
    # look if a replacement is required for this object
    for replacement in regions:
        if re.match("from_" + replacement['object_name'], shape_name):
            # if so, reassign the name with the replacement
            report("Replacing " + shape_name + " with " + replacement['group_name'])
            shape_name = replacement['group_name']
    
    if shape_name not in groups:
        # the group has yet to be created
        #face_group = geompy.CreateGroup(partition, geompy.ShapeType["FACE"])
        #geompy.addToStudyInFather(partition, face_group, shape_name)
        groups[shape_name] = {'shapes': [shape]}
        report("Created group {}".format(shape_name))
    else:
        groups[shape_name]['shapes'].append(shape)
        report("Added to group {}".format(shape_name))

for i in range(len(partition_subshapes)):
    shape = partition_subshapes[i]
    shape_name = shape.GetName()
    assign_group(shape, shape_name)

# create groups
for name in groups:
    group = geompy.CreateGroup(partition, geompy.ShapeType["SOLID"])
    groups[name]['group'] = group

    group_shapes = groups[name]['shapes']
    geompy.UnionList(group, group_shapes)
    geompy.addToStudyInFather(partition, group, name)
    report("Added group {} to study".format(name))